@@ -27,6 +27,21 @@ namespace Aws
                 return aws_checksums_crc64nvme_ex(input.ptr, input.len, previousCRC64NVME);
             }
 
+            uint32_t CombineCRC32(uint32_t crc1, uint32_t crc2, uint64_t len2) noexcept
+            {
+                return aws_checksums_crc32_combine(crc1, crc2, len2);
+            }
+
+            uint32_t CombineCRC32C(uint32_t crc1, uint32_t crc2, uint64_t len2) noexcept
+            {
+                return aws_checksums_crc32c_combine(crc1, crc2, len2);
+            }
+
+            uint64_t CombineCRC64NVME(uint64_t crc1, uint64_t crc2, uint64_t len2) noexcept
+            {
+                return aws_checksums_crc64nvme_combine(crc1, crc2, len2);
+            }
+
         } // namespace Checksum
     } // namespace Crt
 } // namespace Aws
